/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.application.template.logging.oracle;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Iterator;

import org.apache.log4j.spi.ErrorCode;
import org.apache.log4j.spi.LoggingEvent;

import org.jdesigner.platform.db.connection.ConnectionManager;

/**
 * OracleJDBCAppender
 * 
 * @author leesangboo
 * 
 */
public class OracleJDBCAppender extends org.apache.log4j.AppenderSkeleton implements org.apache.log4j.Appender {

	/**
	 * URL of the DB for default connection handling
	 */
	protected String url = "jdbc:odbc:myDB";

	/**
	 * User to connect as for default connection handling
	 */
	protected String username = "me";

	/**
	 * User to use for default connection handling
	 */
	protected String password = "mypassword";

	/**
	 * datasource jndi name
	 */
	protected String datasource = null;

	/**
	 * log table name
	 */
	protected String table = "PLF_SYSTEM_LOG";

	/**
	 * connection
	 */
	protected Connection connection = null;

	/**
	 * size of LoggingEvent buffer before writting to the database. Default is 1.
	 */
	protected int bufferSize = 1;

	/**
	 * ArrayList holding the buffer of Logging Events.
	 */
	protected ArrayList buffer;

	/**
	 * Helper object for clearing out the buffer
	 */
	protected ArrayList removes;

	public OracleJDBCAppender() {
		super();
		buffer = new ArrayList(bufferSize);
		removes = new ArrayList(bufferSize);
	}

	/**
	 * Adds the event to the buffer. When full the buffer is flushed.
	 */
	public void append(LoggingEvent event) {
		buffer.add(event);
		if (buffer.size() >= bufferSize) {
			flushBuffer();
		}
	}

	/**
	 * By default getLogStatement sends the event to the required Layout object. The layout will format the given pattern into a workable SQL string.
	 * 
	 * Overriding this provides direct access to the LoggingEvent when constructing the logging statement.
	 * 
	 */
	protected String getLogStatement(LoggingEvent event) {
		String statement = "";
		try {
			OracleSQLHandler sqlHandler = new OracleSQLHandler();
			statement = sqlHandler.getStatement(table, event);
		} catch (Exception ex) {
			System.out.println(ex);
		}
		return statement;
	}

	/**
	 * 
	 * Override this to provide an alertnate method of getting connections (such as caching). One method to fix this is to open connections at the
	 * start of flushBuffer() and close them at the end. I use a connection pool outside of JDBCAppender which is accessed in an override of this
	 * method.
	 * */
	protected void execute(String sql) throws SQLException {
		Connection con = null;
		Statement stmt = null;
		try {
			con = getConnection();
			stmt = con.createStatement();
			stmt.executeUpdate(sql);
		} catch (SQLException e) {
			if (stmt != null) {
				stmt.close();
			}
			throw e;
		}
		stmt.close();
		closeConnection(con);
	}

	/**
	 * Override this to return the connection to a pool, or to clean up the resource.
	 * 
	 * The default behavior holds a single connection open until the appender is closed (typically when garbage collected).
	 */
	protected void closeConnection(Connection con) {
	}

	/**
	 * Override this to link with your connection pooling system.
	 * 
	 * By default this creates a single connection which is held open until the object is garbage collected.
	 */
	protected Connection getConnection() throws SQLException {
		if (datasource != null) {
			if (connection == null) {
				connection = new ConnectionManager().getConnection(datasource);
			}
		} else {
			if (connection == null) {
				connection = new ConnectionManager().getConnection();
			}
		}
		//
		return connection;
	}

	/**
	 * Closes the appender, flushing the buffer first then closing the default connection if it is open.
	 */
	public void close() {
		flushBuffer();
		try {
			if (connection != null && !connection.isClosed()) {
				connection.close();
			}
		} catch (SQLException e) {
			errorHandler.error("Error closing connection", e, ErrorCode.GENERIC_FAILURE);
		}
		this.closed = true;
	}

	/**
	 * loops through the buffer of LoggingEvents, gets a sql string from getLogStatement() and sends it to execute(). Errors are sent to the
	 * errorHandler.
	 * 
	 * If a statement fails the LoggingEvent stays in the buffer!
	 */
	public void flushBuffer() {
		//Do the actual logging
		removes.ensureCapacity(buffer.size());
		for (Iterator i = buffer.iterator(); i.hasNext();) {
			String sql = "";
			try {
				LoggingEvent logEvent = (LoggingEvent) i.next();
				sql = getLogStatement(logEvent);
				execute(sql);
				removes.add(logEvent);
			} catch (SQLException e) {
				errorHandler.error("Failed to excute sql : " + sql, e, ErrorCode.FLUSH_FAILURE);
			}
		}
		// remove from the buffer any events that were reported
		buffer.removeAll(removes);
		// clear the buffer of reported events
		removes.clear();
	}

	/** closes the appender before disposal */
	public void finalize() {
		close();
	}

	/**
	 * JDBCAppender requires a layout.
	 * */
	public boolean requiresLayout() {
		return true;
	}

	public void setBufferSize(int newBufferSize) {
		bufferSize = newBufferSize;
		buffer.ensureCapacity(bufferSize);
		removes.ensureCapacity(bufferSize);
	}

	public int getBufferSize() {
		return bufferSize;
	}

	public String getTable() {
		return table;
	}

	public void setTable(String table) {
		this.table = table;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getUrl() {
		return url;
	}

	public void setUrl(String url) {
		this.url = url;
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getDatasource() {
		return datasource;
	}

	public void setDatasource(String datasource) {
		this.datasource = datasource;
	}

	/**
	 * Ensures that the given driver class has been loaded for sql connection creation.
	 */
	public void setDriver(String driverClass) {
		try {
			Class.forName(driverClass);
		} catch (Exception e) {
			errorHandler.error("Failed to load driver", e, ErrorCode.GENERIC_FAILURE);
		}
	}

}
