/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.application.template.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jdesigner.platform.core.threadlocal.ThreadLocalProperties;
import org.jdesigner.platform.data.Page;
import org.jdesigner.platform.util.StringUtils;

import org.jdesigner.application.template.dao.SecurityDAO;
import org.jdesigner.application.template.domain.UserLogin;
import org.jdesigner.application.template.service.CommonServiceException;
import org.jdesigner.application.template.service.CommonServiceProperties;
import org.jdesigner.application.template.service.LoginEventListener;
import org.jdesigner.application.template.service.SecurityService;
import org.jdesigner.application.template.service.provider.PasswordStringEncrypter;
import org.jdesigner.application.template.web.WebKeys;

/**
 * Security Service Impl
 * 
 * @author leesangboo
 * 
 */
public class SecurityServiceImpl implements SecurityService {

	private static final Log log = LogFactory.getLog(SecurityServiceImpl.class);

	public static final String SECURITY_PASS_PHRASE_KEY = "service.security.password.chiperkey";

	public static final String LOGIN_EVENT_LISTENER_KEY = "service.security.login.event.listener";

	private SecurityDAO dao;

	/**
	 * SecurityServiceImpl
	 * 
	 * @param session
	 */
	public SecurityServiceImpl() {
		this.dao = new SecurityDAO();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.jdesigner.common.services.SecurityService#getRoles()
	 */
	public ArrayList getRoles() {
		return getUserLogin().getRoles();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.jdesigner.common.services.SecurityService#getUserLogin()
	 */
	public UserLogin getUserLogin() {
		return (UserLogin) ThreadLocalProperties.get(THREAD_LOCAL_USER_LOGIN_KEY);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.application.template.service.SecurityService#getUserLoginFromSession(javax.servlet.http.HttpSession)
	 */
	public UserLogin getUserLoginFromSession(HttpSession session) {
		return (UserLogin) session.getAttribute(WebKeys.USER_LOGIN_KEY);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.jdesigner.common.services.SecurityService#isUserAuthenticated()
	 */
	public boolean isUserAuthenticated() {
		return getUserLogin() != null && StringUtils.isNotEmpty(getUserLogin().getUsername()) ? true : false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.jdesigner.common.services.SecurityService#isUserInRole(java.lang .String)
	 */
	public boolean isUserInRole(String role) {
		if (StringUtils.isEmpty(role)) {
			return false;
		}
		return getUserLogin().hasRole(role);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.application.template.service.SecurityService#isUserInRoles (java.util.List)
	 */
	public boolean isUserInRoles(List roles) {
		if (roles == null || roles.size() < 1) {
			return false;
		}
		boolean accessable = false;
		for (int i = 0; roles != null && i < roles.size(); i++) {
			String role = (String) roles.get(i);
			// List user_roles = getUserLogin().getRoles();
			// log.info("access_role=" + role);
			// log.info("user_roles=" + user_roles);
			accessable = getUserLogin().hasRole(role);
			if (accessable) {
				break;
			}
		}
		return accessable;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.application.template.service.SecurityService#login(java .lang.String, java.lang.String, java.lang.String)
	 */
	public boolean login(HttpSession session, String ip, String username, String password) {
		//
		String encryptedPassword = this.encryptPassword(password);
		//
		log.info("encryptedPassword=" + encryptedPassword);
		//
		boolean isLogin = dao.checkLogin(username, encryptedPassword);
		if (isLogin) {
			this.setUserLogin(session, ip, username, password);
			log.info("Login success : username=[" + username + "]");
		} else {
			log.info("Login failed.");
		}
		return isLogin;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.jdesigner.application.template.service.SecurityService# setUserLoinInSession(java.lang.String, java.lang.String, java.lang.String)
	 */
	public void setUserLogin(HttpSession session, String ip, String username, String password) {
		ArrayList roles = (ArrayList) dao.getUserRoles(username);
		if (roles == null) {
			roles = new ArrayList();
			log.warn("Roles is null : username=[" + username + "]");
		}
		//
		UserLogin user_login = new UserLogin(ip, username, password, roles);
		session.setAttribute(WebKeys.USER_LOGIN_KEY, user_login);
		//
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.jdesigner.common.services.SecurityService#logout(java.lang.String)
	 */
	public void logout(HttpSession session) {
		session.removeAttribute(WebKeys.USER_LOGIN_KEY);
		session.invalidate();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.doosan.oracle.ebs.common.service.SecurityService#selectSecurityGroupPage (java.util.Map, int, int, java.lang.String, boolean)
	 */
	public Page selectAuthGroupPage(Map params, int start, int range, String sort, boolean desc) throws CommonServiceException {
		return this.dao.selectAuthGroupPage(params, start, range, sort, desc);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.jdesigner.application.template.service.SecurityService# selectAuthRolePage(java.util.Map, int, int, java.lang.String, boolean)
	 */
	public Page selectAuthRolePage(Map params, int start, int range, String sort, boolean desc) throws CommonServiceException {
		return this.dao.selectAuthRolePage(params, start, range, sort, desc);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.application.template.service.SecurityService#selectAuthGroup (java.util.Map)
	 */
	public Map selectAuthGroup(Map params) throws CommonServiceException {
		return this.dao.selectAuthGroup(params);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.application.template.service.SecurityService#insertAuthGroup (java.util.Map, java.lang.String[])
	 */
	public void insertAuthGroup(Map params, List auth_role_list) throws CommonServiceException {
		this.dao.insertAuthGroup(params, auth_role_list);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.application.template.service.SecurityService#updateAuthGroup (java.util.Map, java.lang.String[])
	 */
	public void updateAuthGroup(Map params, List auth_role_list) throws CommonServiceException {
		this.dao.updateAuthGroup(params, auth_role_list);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.application.template.service.SecurityService#deleteAuthGroup (java.util.Map, java.lang.String[])
	 */
	public void deleteAuthGroup(Map params) throws CommonServiceException {
		this.dao.deleteAuthGroup(params);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.jdesigner.application.template.service.SecurityService# selectAuthGroupRoleList(java.util.Map)
	 */
	public List selectAuthGroupRoleList(Map params) throws CommonServiceException {
		return this.dao.selectAuthGroupRoleList(params);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.application.template.service.SecurityService#selectAuthRole (java.util.Map)
	 */
	public Map selectAuthRole(Map params) throws CommonServiceException {
		return this.dao.selectAuthRole(params);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.application.template.service.SecurityService#insertAuthRole (java.util.Map)
	 */
	public void insertAuthRole(Map params) throws CommonServiceException {
		this.dao.insertAuthRole(params);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.application.template.service.SecurityService#updateAuthRole (java.util.Map)
	 */
	public void updateAuthRole(Map params) throws CommonServiceException {
		this.dao.updateAuthRole(params);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.application.template.service.SecurityService#deleteAuthRole (java.util.Map)
	 */
	public void deleteAuthRole(Map params) throws CommonServiceException {
		this.dao.deleteAuthRole(params);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.jdesigner.application.template.service.SecurityService# insertLoginHistory(java.util.Map)
	 */
	public void insertLoginHistory(Map params) throws CommonServiceException {
		this.dao.insertLoginHistory(params);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.application.template.service.SecurityService#encryptPassword (java.lang.String)
	 */
	public String encryptPassword(String password) {
		PasswordStringEncrypter desEncrypter = new PasswordStringEncrypter(getSecurityPassPhrase());
		String encryptedPassword = desEncrypter.encrypt(password);
		return encryptedPassword;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.application.template.service.SecurityService#decryptPassword (java.lang.String)
	 */
	public String decryptPassword(String encryptedPassword) {
		PasswordStringEncrypter desEncrypter = new PasswordStringEncrypter(getSecurityPassPhrase());
		String decryptedPassword = desEncrypter.decrypt(encryptedPassword);
		return decryptedPassword;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.jdesigner.application.template.service.SecurityService# getLoginEventListener()
	 */
	public LoginEventListener getLoginEventListener() throws CommonServiceException {
		LoginEventListener listener = null;
		try {
			String listenerClassName = CommonServiceProperties.getString(LOGIN_EVENT_LISTENER_KEY);
			listener = (LoginEventListener) Class.forName(listenerClassName).newInstance();
		} catch (Exception ex) {
			throw new CommonServiceException("Login event listener getting failed : " + ex.getMessage(), ex);
		}
		return listener;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.jdesigner.application.template.service.SecurityService# selectUserLoginHistoryByLast(java.util.Map)
	 */
	public Map selectUserLoginHistoryByLast(Map params) throws CommonServiceException {
		params.put("username", this.getUserLogin().getUsername());
		return dao.selectUserLoginHistoryByLast(params);
	}

	/**
	 * SecurityPassPhrase
	 * 
	 * @return
	 */
	private String getSecurityPassPhrase() {
		return CommonServiceProperties.getString(SECURITY_PASS_PHRASE_KEY);
	}

}
