/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.application.template.web.filter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jdesigner.platform.core.threadlocal.ThreadLocalProperties;
import org.jdesigner.platform.util.PatternMatchUtils;
import org.jdesigner.platform.util.StringUtils;

import org.jdesigner.application.template.domain.SecurityAccessRule;
import org.jdesigner.application.template.domain.UserLogin;
import org.jdesigner.application.template.service.CommonServiceFactory;
import org.jdesigner.application.template.service.SecurityService;
import org.jdesigner.application.template.service.config.SecurityConfigurationLoader;
import org.jdesigner.application.template.service.factory.SecurityAccessSingletoneFactory;
import org.jdesigner.application.template.web.WebKeys;

/**
 * SecurityAccessFilter
 * 
 * @author leesangboo
 * 
 */
public class SecurityFilter implements Filter {

	private static final Log log = LogFactory.getLog(SecurityFilter.class);

	//private static final String CONFIG_PATH = "/WEB-INF/security-config.xml";

	private static final String CONFIG_PATH_KEY = "config-path";
	private static final String LOGIN_ERROR_PAGE_KEY = "login-error-page";
	private static final String ROLES_ERROR_PAGE_KEY = "roles-error-page";

	private String configPath;
	private String loginErrorPage;
	private String rolesErrorPage;

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	public void init(FilterConfig config) throws ServletException {
		configPath = config.getInitParameter(CONFIG_PATH_KEY);
		loginErrorPage = config.getInitParameter(LOGIN_ERROR_PAGE_KEY);
		rolesErrorPage = config.getInitParameter(ROLES_ERROR_PAGE_KEY);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.Filter#destroy()
	 */
	public void destroy() {
		//
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
	 */
	public void doFilter(ServletRequest srequest, ServletResponse sresponse, FilterChain chain) throws IOException, ServletException {

		HttpServletRequest request = (HttpServletRequest) srequest;
		HttpServletResponse response = (HttpServletResponse) sresponse;
		HttpSession session = request.getSession();

		try {

			String contextPath = request.getContextPath();
			String uri = request.getRequestURI().replace(request.getContextPath(), "");

			//
			log.debug("Security filter for URI = [" + uri + "]");

			//
			UserLogin userLogin = (UserLogin) session.getAttribute(WebKeys.USER_LOGIN_KEY);
			if (userLogin != null) {
				log.debug("UserLogin set thread local properties.");
				ThreadLocalProperties.put(SecurityService.THREAD_LOCAL_USER_LOGIN_KEY, userLogin);
			}

			//
			SecurityService securityService = CommonServiceFactory.getSecurityService();

			//
			SecurityAccessRule securityAccessRule = getSecurityAccessRule(uri);

			String fromuri = request.getRequestURI();
			String param = (StringUtils.isNotEmpty(request.getQueryString())) ? ("?" + request.getQueryString()) : "";
			if (securityAccessRule != null) {

				if (securityAccessRule.isAll()) {
					chain.doFilter(srequest, sresponse);
				} else if (securityAccessRule.isLogin()) {
					if (securityService.isUserAuthenticated()) {
						chain.doFilter(srequest, sresponse);
					} else {
						FilterUtils.redirectAndReturn(response, contextPath + loginErrorPage + "?redirect=" + fromuri + param);
					}
				} else if (securityAccessRule.getRoles() != null && !securityAccessRule.getRoles().isEmpty() && securityAccessRule.getRoles().size() > 0) {
					if (securityService.isUserAuthenticated()) {
						boolean hasRole = false;
						List ruleRoles = securityAccessRule.getRoles();
						for (int i = 0; i < ruleRoles.size(); i++) {
							if (securityService.isUserInRole((String) ruleRoles.get(i))) {
								hasRole = true;
								break;
							}
						}

						if (hasRole) {
							chain.doFilter(srequest, sresponse);
						} else {
							FilterUtils.redirectAndReturn(response, contextPath + rolesErrorPage);
						}
					} else {
						FilterUtils.redirectAndReturn(response, contextPath + loginErrorPage + "?redirect=" + fromuri + param);
					}
				}

			} else {
				chain.doFilter(srequest, sresponse);
			}

		} catch (Exception ex) {
			log.error("Security filter error : " + ex.getMessage(), ex);
			throw new ServletException(ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * 
	 * (java.lang.String)
	 */
	private SecurityAccessRule getSecurityAccessRule(String uri) throws Exception {
		SecurityAccessRule securityAccessRule = null;
		try {
			SecurityAccessSingletoneFactory factory = SecurityAccessSingletoneFactory.getInstance();
			List securityAccessRuleList = factory.getSecurityAccessRuleList();
			if (securityAccessRuleList == null) {
				SecurityConfigurationLoader loader = new SecurityConfigurationLoader();
				securityAccessRuleList = loader.load(configPath);
				if (securityAccessRuleList != null && securityAccessRuleList.size() > 1) {
					factory.setSecurityAccessRuleList(securityAccessRuleList);
				} else {
					factory.setSecurityAccessRuleList(new ArrayList());
				}
			}
			for (int i = 0; i < securityAccessRuleList.size(); i++) {
				SecurityAccessRule rule = (SecurityAccessRule) securityAccessRuleList.get(i);
				if (PatternMatchUtils.simpleMatch(rule.getUrlPattern(), uri)) {
					securityAccessRule = rule;
				}
			}
		} catch (Exception ex) {
			log.error("Getting error seucrity acess rule : " + ex.getMessage(), ex);
			throw ex;
		}
		return securityAccessRule;
	}

}
