/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.platform.db.processor.impl;

import java.sql.Connection;
import java.util.List;
import java.util.Map;

import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jdesigner.platform.db.connection.ConnectionUtils;
import org.jdesigner.platform.db.logger.SQLLogger;
import org.jdesigner.platform.db.parser.SQLParser;
import org.jdesigner.platform.db.parser.SQLParserFactory;
import org.jdesigner.platform.db.provider.RDBMSProvider;
import org.jdesigner.platform.db.provider.RDBMSProviderFactory;
import org.jdesigner.platform.db.query.NamedQuery;
import org.jdesigner.platform.db.query.Query;
import org.jdesigner.platform.db.query.TableQuery;
import org.jdesigner.platform.exception.DatabaseException;

/**
 * <B>Processor Support</B>
 * 
 * @author leesangboo
 * 
 */
abstract public class BaseProcessor {

	private static final Log log = LogFactory.getLog(BaseProcessor.class);

	/**
	 * AutoCommit을 true로 설정하고, Connection을 해제한다.
	 * 
	 * @param conn
	 * @throws DatabaseException
	 */
	protected void closeConnectionForInactiveTransaction(Connection conn) throws DatabaseException {
		if (!ConnectionUtils.isAutoCommit(conn)) {
			ConnectionUtils.commitConnection(conn);
			ConnectionUtils.setAutoCommit(conn, true);
			log.debug("Connection commit and setAutoCommit(true);");
		}
		ConnectionUtils.closeConnection(conn);
	}

	/**
	 * QueryRunner를 반환한다.
	 * 
	 * @return
	 */
	protected QueryRunner getQueryRunner() {
		return new QueryRunner(true);
	}

	/**
	 * QueryRunner를 반환한다.
	 * 
	 * @param pmdKnownBroken
	 * @return
	 */
	protected QueryRunner getQueryRunner(boolean pmdKnownBroken) {
		return new QueryRunner(pmdKnownBroken);
	}

	/**
	 * NamedQueryRunner를 반환한다.
	 * 
	 * @param pmdKnownBroken
	 * @return
	 */
	protected NamedQueryRunner getNamedQueryRunner() {
		return new NamedQueryRunner(true);
	}

	/**
	 * NamedQueryRunner를 반환한다.
	 * 
	 * @param pmdKnownBroken
	 * @return
	 */
	protected NamedQueryRunner getNamedQueryRunner(boolean pmdKnownBroken) {
		return new NamedQueryRunner(pmdKnownBroken);
	}

	/**
	 * PagingSQLHandler를 반환한다.
	 * 
	 * @return
	 * @throws DatabaseException
	 */
	protected RDBMSProvider getRDBMSProvider(Connection connection) throws DatabaseException {
		return RDBMSProviderFactory.getInstance().getRDBMSProvider(connection);
	}

	/**
	 * Query로 부터 Dynamic SQL을 반환한다.
	 * 
	 * @param query
	 * @return
	 * @throws DatabaseException
	 */
	protected String getParsedDynamicSQL(Query query) throws DatabaseException {
		SQLParser parser = SQLParserFactory.getQueryParser(query);
		return parser.parseSQL(query, "");
	}

	/**
	 * Query로 부터 Dynamic SQL을 반환한다.
	 * 
	 * @param query
	 * @return
	 * @throws DatabaseException
	 */
	protected String getParsedDynamicSQL(NamedQuery query) throws DatabaseException {
		SQLParser parser = SQLParserFactory.getQueryParser(query);
		return parser.parseSQL(query, "");
	}

	/**
	 * Query로 부터 Dynamic SQL을 반환한다.
	 * 
	 * @param query
	 * @return
	 * @throws DatabaseException
	 */
	protected String getParsedDynamicSQL(TableQuery query, String mode) throws DatabaseException {
		SQLParser parser = SQLParserFactory.getQueryParser(query);
		return parser.parseSQL(query, mode);
	}

	/**
	 * 파라메터를 생성한다.
	 * 
	 * @param parameters
	 * @return
	 * @throws DatabaseException
	 */
	protected Object[] createListParameters(Query query) throws DatabaseException {
		return query.getParameters() != null ? query.getParameters().toArray() : null;
	}

	/**
	 * 배치 파라메터를 생성한다.
	 * 
	 * @param parameters
	 * @return
	 * @throws DatabaseException
	 */
	protected Object[][] createListBatchParameters(Query query) throws DatabaseException {
		return null;
	}

	/**
	 * 배치 파라메터를 생성한다.
	 * 
	 * @param parameters
	 * @return
	 * @throws DatabaseException
	 */
	protected List createMapBatchParameters(NamedQuery query) throws DatabaseException {
		return null;
	}

	/**
	 * 파라메터를 생성한다.
	 * 
	 * @param parameters
	 * @return
	 * @throws DatabaseException
	 */
	protected Map createMapParameters(NamedQuery query) throws DatabaseException {
		return query.getParameters() != null ? query.getParameters() : null;
	}

	/**
	 * 배치 파라메터를 생성한다.
	 * 
	 * @param parameters
	 * @return
	 * @throws DatabaseException
	 */
	protected List createMapBatchParameters(TableQuery query) throws DatabaseException {
		return null;
	}

	/**
	 * 파라메터를 생성한다.
	 * 
	 * @param parameters
	 * @return
	 * @throws DatabaseException
	 */
	protected Map createMapParameters(TableQuery query) throws DatabaseException {
		return query.getParameters() != null ? query.getParameters() : null;
	}

	/**
	 * SQL을 로깅한다.
	 * 
	 * @param sql
	 * @param params
	 */
	protected void loggingSQL(String sql, Object[] params, boolean inTransaction) {
		SQLLogger logger = new SQLLogger();
		logger.log(sql, params, inTransaction);
	}

	/**
	 * SQL을 로깅한다.
	 * 
	 * @param sql
	 * @param params
	 */
	protected void loggingSQL(String sql, List params, boolean inTransaction) {
		SQLLogger logger = new SQLLogger();
		logger.log(sql, (params != null) ? params.toArray() : null, inTransaction);
	}

	/**
	 * SQL을 로깅한다.
	 * 
	 * @param sql
	 * @param params
	 */
	protected void loggingSQL(String sql, Map params, boolean inTransaction) {
		SQLLogger logger = new SQLLogger();
		logger.log(sql, params, inTransaction);
	}

}
