/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.platform.db.processor.impl;

import java.sql.Connection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jdesigner.platform.data.Page;
import org.jdesigner.platform.db.domain.UpdateResult;
import org.jdesigner.platform.db.paging.PagingSQLHandler;
import org.jdesigner.platform.db.processor.QueryProcessingException;
import org.jdesigner.platform.db.provider.RDBMSProvider;

/**
 * <B>SQL Processor</B>
 * 
 * @author leesangboo
 * 
 */
abstract public class SQLProcessor extends BaseProcessor {

	private static final Log log = LogFactory.getLog(SQLProcessor.class);

	/**
	 * SELECT SQL을 실행하여 Page 도매인을 반환한다.
	 * 
	 * @param conn
	 * @param sql
	 * @param params
	 * @param start
	 * @param range
	 * @return
	 * @throws QueryProcessingException
	 */
	public Page selectPage(Connection conn, boolean inTransaction, String sql, Object[] params, int start, int range, String column, boolean desc) throws QueryProcessingException {

		Page page = null;

		try {

			RDBMSProvider provider = getRDBMSProvider(conn);
			//
			PagingSQLHandler pagingSQL = provider.getPagingSQLHandler();
			sql = pagingSQL.generatePagingSQL(sql, column, desc);
			params = pagingSQL.createParameter(params, start, range);

			//
			params = provider.getParameterBinder().bind(conn, params);

			//
			super.loggingSQL(sql, params, inTransaction);

			//
			if (params != null) {
				List result = (List) getQueryRunner().query(conn, sql, provider.getMapListResultHandler(), params);
				page = new Page(pagingSQL.getCountFromResult(result), result, start, range);
			} else {
				List result = (List) getQueryRunner().query(conn, sql, provider.getMapListResultHandler());
				page = new Page(pagingSQL.getCountFromResult(result), result, start, range);
			}

		} catch (Exception ex) {
			String msg = "SQL execute failed : " + ex.getMessage();
			log.error(msg);
			throw new QueryProcessingException(msg, ex);
		} finally {
			if (!inTransaction) {
				super.closeConnectionForInactiveTransaction(conn); //
			}
		}
		return page;
	}

	/**
	 * SELECT SQL을 실행하여 Page 도매인을 반환한다.
	 * 
	 * @param conn
	 * @param inTransaction
	 * @param sql
	 * @param params
	 * @param start
	 * @param range
	 * @param column
	 * @param desc
	 * @return
	 * @throws QueryProcessingException
	 */
	public Page selectPage(Connection conn, boolean inTransaction, String sql, Map params, int start, int range, String column, boolean desc) throws QueryProcessingException {
		Page page = null;
		try {

			RDBMSProvider provider = getRDBMSProvider(conn);
			//
			PagingSQLHandler pagingSQL = provider.getPagingSQLHandler();
			sql = pagingSQL.generatePagingNamedSQL(sql, column, desc);
			params = pagingSQL.createNamedParameter(params, start, range);

			//
			params = provider.getParameterBinder().bind(conn, params);

			//
			super.loggingSQL(sql, params, inTransaction);

			//
			List result = (List) getNamedQueryRunner().query(conn, sql, provider.getMapListResultHandler(), params);
			page = new Page(pagingSQL.getCountFromResult(result), result, start, range);

		} catch (Exception ex) {
			String msg = "SQL execute failed : " + ex.getMessage();
			log.error(msg);
			throw new QueryProcessingException(msg, ex);
		} finally {
			if (!inTransaction) {
				super.closeConnectionForInactiveTransaction(conn); //
			}
		}
		return page;
	}

	/**
	 * SELECT SQL을 실행하여 Page 도매인을 반환한다.
	 * 
	 * @param conn
	 * @param inTransaction
	 * @param sql
	 * @param bean
	 * @param start
	 * @param range
	 * @return
	 * @throws QueryProcessingException
	 */
	public Page selectPage(Connection conn, boolean inTransaction, String sql, Object bean, int start, int range, String column, boolean desc) throws QueryProcessingException {
		Page page = null;
		try {

			Map params = org.apache.commons.beanutils.BeanUtils.describe(bean);

			RDBMSProvider provider = getRDBMSProvider(conn);

			//
			PagingSQLHandler pagingSQL = provider.getPagingSQLHandler();
			sql = pagingSQL.generatePagingNamedSQL(sql, column, desc);
			params = pagingSQL.createNamedParameter(params, start, range);

			//
			params = provider.getParameterBinder().bind(conn, params);

			//
			super.loggingSQL(sql, params, inTransaction);

			//
			List result = (List) getNamedQueryRunner().query(conn, sql, provider.getMapListResultHandler(), params);
			page = new Page(pagingSQL.getCountFromResult(result), result, start, range);

		} catch (Exception ex) {
			String msg = "SQL execute failed : " + ex.getMessage();
			log.error(msg);
			throw new QueryProcessingException(msg, ex);
		} finally {
			if (!inTransaction) {
				super.closeConnectionForInactiveTransaction(conn); //
			}
		}
		return page;
	}

	/**
	 * SELECT SQL을 실행하여 List를 반환한다.
	 * 
	 * @param conn
	 * @param inTransaction
	 * @param sql
	 * @param params
	 * @return
	 * @throws QueryProcessingException
	 */
	public List selectList(Connection conn, boolean inTransaction, String sql, Object[] params) throws QueryProcessingException {
		List result = null;
		try {
			//
			RDBMSProvider provider = getRDBMSProvider(conn);
			//
			params = provider.getParameterBinder().bind(conn, params);

			//
			super.loggingSQL(sql, params, inTransaction);

			//
			if (params != null) {
				result = (List) getQueryRunner().query(conn, sql, provider.getMapListResultHandler(), params);
			} else {
				result = (List) getQueryRunner().query(conn, sql, provider.getMapListResultHandler());
			}

		} catch (Exception ex) {
			String msg = "SQL execute failed : " + ex.getMessage();
			log.error(msg);
			throw new QueryProcessingException(msg, ex);
		} finally {
			if (!inTransaction) {
				super.closeConnectionForInactiveTransaction(conn); //
			}
		}
		return result;
	}

	/**
	 * SELECT SQL을 실행하여 List를 반환한다.
	 * 
	 * @param conn
	 * @param inTransaction
	 * @param sql
	 * @param params
	 * @return
	 * @throws QueryProcessingException
	 */
	public List selectList(Connection conn, boolean inTransaction, String sql, Map params) throws QueryProcessingException {
		List result = null;
		try {
			//
			RDBMSProvider provider = getRDBMSProvider(conn);
			//
			params = provider.getParameterBinder().bind(conn, params);
			//
			super.loggingSQL(sql, params, inTransaction);
			//
			result = (List) getNamedQueryRunner().query(conn, sql, provider.getMapListResultHandler(), params);
		} catch (Exception ex) {
			String msg = "SQL execute failed : " + ex.getMessage();
			log.error(msg);
			throw new QueryProcessingException(msg, ex);
		} finally {
			if (!inTransaction) {
				super.closeConnectionForInactiveTransaction(conn); //
			}
		}
		return result;
	}

	/**
	 * SELECT SQL을 실행하여 Map을 반환한다.
	 * 
	 * @param conn
	 * @param inTransaction
	 * @param sql
	 * @param params
	 * @return
	 * @throws QueryProcessingException
	 */
	public Map selectMap(Connection conn, boolean inTransaction, String sql, Object[] params) throws QueryProcessingException {
		Map result = null;
		try {
			//
			RDBMSProvider provider = getRDBMSProvider(conn);
			//
			super.loggingSQL(sql, params, inTransaction);
			//
			result = (Map) getQueryRunner().query(conn, sql, provider.getMapResultHandler(), params);
		} catch (Exception ex) {
			String msg = "SQL execute failed : " + ex.getMessage();
			log.error(msg);
			throw new QueryProcessingException(msg, ex);
		} finally {
			if (!inTransaction) {
				super.closeConnectionForInactiveTransaction(conn); //
			}
		}
		return result;
	}

	/**
	 * SELECT SQL을 실행하여 Map을 반환한다.
	 * 
	 * @param sql
	 * @param params
	 * @return
	 * @throws QueryProcessingException
	 */
	public Map selectMap(Connection conn, boolean inTransaction, String sql, Map params) throws QueryProcessingException {
		Map result = null;
		try {
			//
			RDBMSProvider provider = getRDBMSProvider(conn);

			//
			super.loggingSQL(sql, params, inTransaction);

			//
			result = (Map) getNamedQueryRunner().query(conn, sql, provider.getMapResultHandler(), params);

		} catch (Exception ex) {
			String msg = "SQL execute failed : " + ex.getMessage();
			log.error(msg);
			throw new QueryProcessingException(msg, ex);
		} finally {
			if (!inTransaction) {
				super.closeConnectionForInactiveTransaction(conn);
			}
		}
		return result;
	}

	/**
	 * SELECT SQL을 실행하여 Object 을 반환한다.
	 * 
	 * @param conn
	 * @param inTransaction
	 * @param sql
	 * @param params
	 * @return
	 * @throws QueryProcessingException
	 */
	public Object selectObject(Connection conn, boolean inTransaction, String sql, Object[] params) throws QueryProcessingException {
		Object result = null;
		try {

			RDBMSProvider provider = getRDBMSProvider(conn);

			//
			super.loggingSQL(sql, params, inTransaction);

			//
			result = getQueryRunner().query(conn, sql, provider.getObjectResultHandler(), params);

		} catch (Exception ex) {
			String msg = "SQL execute failed : " + ex.getMessage();
			log.error(msg);
			throw new QueryProcessingException(msg, ex);
		} finally {
			if (!inTransaction) {
				super.closeConnectionForInactiveTransaction(conn); //
			}
		}
		return result;
	}

	/**
	 * SELECT SQL을 실행하여 Object 을 반환한다.
	 * 
	 * @param sql
	 * @param params
	 * @return
	 * @throws QueryProcessingException
	 */
	public Object selectObject(Connection conn, boolean inTransaction, String sql, Map params) throws QueryProcessingException {
		Object result = null;
		try {
			//
			RDBMSProvider provider = getRDBMSProvider(conn);
			//
			super.loggingSQL(sql, params, inTransaction);
			//
			result = getNamedQueryRunner().query(conn, sql, provider.getObjectResultHandler(), params);
		} catch (Exception ex) {
			String msg = "SQL execute failed : " + ex.getMessage();
			log.error(msg);
			throw new QueryProcessingException(msg, ex);
		} finally {
			if (!inTransaction) {
				super.closeConnectionForInactiveTransaction(conn);
			}
		}
		return result;
	}

	/**
	 * INSERT, UPDATE, DELETE SQL을 실행하여 처리된 행의 갯수를 반환한다.
	 * 
	 * @param conn
	 * @param inTransaction
	 * @param sql
	 * @param params
	 * @return
	 * @throws QueryProcessingException
	 */
	public UpdateResult update(Connection conn, boolean inTransaction, String sql, Object[] params) throws QueryProcessingException {
		UpdateResult result = null;
		try {
			//
			RDBMSProvider provider = getRDBMSProvider(conn);
			//
			params = provider.getParameterBinder().bind(conn, params);
			//
			super.loggingSQL(sql, params, inTransaction);
			//
			int count = getQueryRunner().update(conn, sql, params);
			Map data = new HashMap();
			for (int i = 0; params != null && i < params.length; i++) {
				data.put(String.valueOf((i + i)), params[i]);
			}
			result = new UpdateResult(count, data);
		} catch (Exception ex) {
			String msg = "SQL execute failed : " + ex.getMessage();
			log.error(msg);
			throw new QueryProcessingException(msg, ex);
		} finally {
			if (!inTransaction) {
				super.closeConnectionForInactiveTransaction(conn); //
			}
		}
		return result;
	}

	/**
	 * INSERT, UPDATE, DELETE SQL을 실행하여 처리된 행의 갯수를 반환한다.
	 * 
	 * @param conn
	 * @param inTransaction
	 * @param sql
	 * @param params
	 * @return
	 * @throws QueryProcessingException
	 */
	public UpdateResult update(Connection conn, boolean inTransaction, String sql, Map params) throws QueryProcessingException {
		UpdateResult result = null;
		try {

			//
			RDBMSProvider provider = getRDBMSProvider(conn);
			//
			params = provider.getParameterBinder().bind(conn, params);

			//
			super.loggingSQL(sql, params, inTransaction);

			//
			int count = getNamedQueryRunner().update(conn, sql, params);
			result = new UpdateResult(count, params);

		} catch (Exception ex) {
			String msg = "SQL execute failed : " + ex.getMessage();
			log.error(msg);
			throw new QueryProcessingException(msg, ex);
		} finally {
			if (!inTransaction) {
				super.closeConnectionForInactiveTransaction(conn);
			}
		}
		return result;
	}

	/**
	 * Batch를 실행하여 처리된 행의 갯수 배열을 반환한다.
	 * 
	 * @param conn
	 * @param inTransaction
	 * @param sql
	 * @param params
	 * @return
	 * @throws QueryProcessingException
	 */
	public int[] batch(Connection conn, boolean inTransaction, String sql, Object[][] params) throws QueryProcessingException {

		int[] result;

		try {

			//
			super.loggingSQL(sql, params, inTransaction);

			//
			result = getQueryRunner().batch(conn, sql, params);

		} catch (Exception ex) {
			String msg = "SQL execute failed : " + ex.getMessage();
			log.error(msg);
			throw new QueryProcessingException(msg, ex);
		} finally {
			if (!inTransaction) {
				super.closeConnectionForInactiveTransaction(conn); //
			}
		}
		return result;
	}

	/**
	 * Batch를 실행하여 처리된 행의 갯수 배열을 반환한다.
	 * 
	 * @param conn
	 * @param inTransaction
	 * @param sql
	 * @param params
	 * @return
	 * @throws QueryProcessingException
	 */
	public int[] batch(Connection conn, boolean inTransaction, String sql, List params) throws QueryProcessingException {
		int[] result = null;
		try {

			//
			super.loggingSQL(sql, params, inTransaction);

			//
			result = getNamedQueryRunner().batch(conn, sql, params);

		} catch (Exception ex) {
			String msg = "SQL execute failed : " + ex.getMessage();
			log.error(msg);
			throw new QueryProcessingException(msg, ex);
		} finally {
			super.closeConnectionForInactiveTransaction(conn);
		}
		return result;
	}

}
