/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.platform.db.query.impl;

import java.util.HashMap;
import java.util.Map;

import org.jdesigner.platform.db.processor.TableQueryProcessor;
import org.jdesigner.platform.db.processor.impl.TableQueryProcessorImpl;
import org.jdesigner.platform.db.query.TableQuery;
import org.jdesigner.platform.db.session.Session;

/**
 * <B>Table Query</B>
 * 
 * 
 * @author leesangboo
 * 
 */
public class TableQueryImpl extends SessionQuery implements TableQuery {

	/**
	 * 테이블 명
	 */
	private String tableName; //

	/**
	 * 프라이머리 키 배열
	 */
	private String[] primaryKeys; // 

	/**
	 * INSERT/UPDATE 시 처리 대상이 될 컬럼명 배열</P> 이 값은 모든 컬럼이 처리 대상일 경우는 상관없지만, 특정 컬럼만 프로세싱 할 경우 설정하면 된다.</P> 파라메터가 java.util.Map으로 구성되므로 Map의 Key Set을 반환하면
	 * 기본적으로 Map에 설정된 값이 설정된 키 컬럼만 처리된다.</P>
	 */
	private String[] processTargetColumns; //

	/**
	 * 파라메터 맵속의 Key만 INSERT/UPDATE 컬럼으로 바인딩</P> 이 값을 설정하면 자동으로 processTargetColumns값이 Parameter Map의 Key Array로 설정된다.</P>
	 */
	private boolean onlyParamKeyBinding = true;

	/**
	 * 바인드 파라메터
	 */
	private Map parameters;

	/**
	 * Query 생성자 (TABLE)
	 * 
	 * @param tableName
	 *            - 테이블명
	 * @param mode
	 *            - 쿼리 모드
	 */
	public TableQueryImpl(Session session, String tableName) {
		super.setSession(session);
		this.tableName = tableName;
	}

	/**
	 * Query 생성자 (TABLE)
	 * 
	 * @param tableName
	 *            - 테이블명
	 * @param mode
	 *            - 쿼리 모드
	 */
	public TableQueryImpl(Session session, String tableName, Map parameters) {
		super.setSession(session);
		this.tableName = tableName;
		this.parameters = parameters;
	}

	/**
	 * Query 생성자 (TABLE)
	 * 
	 * @param tableName
	 *            - 테이블명
	 * @param mode
	 *            - 쿼리 모드
	 * @param primaryKeys
	 *            - 프라이머리 키 배열
	 */
	public TableQueryImpl(Session session, String tableName, String[] primaryKeys, Map parameters) {
		super.setSession(session);
		this.tableName = tableName;
		this.primaryKeys = primaryKeys;
		this.parameters = parameters;
	}

	/**
	 * Query 생성자 (TABLE)
	 * 
	 * @param transaction
	 * @param connection
	 * @param tableName
	 * @param parameters
	 * @param onlyParamKeyBinding
	 */
	public TableQueryImpl(Session session, String tableName, Map parameters, boolean onlyParamKeyBinding) {
		super.setSession(session);
		this.tableName = tableName;
		this.onlyParamKeyBinding = onlyParamKeyBinding;
		this.parameters = parameters;
	}

	/**
	 * Query 생성자 (TABLE)
	 * 
	 * @param transaction
	 * @param connection
	 * @param tableName
	 * @param primaryKeys
	 * @param parameters
	 * @param onlyParamKeyBinding
	 */
	public TableQueryImpl(Session session, String tableName, String[] primaryKeys, Map parameters, boolean onlyParamKeyBinding) {
		super.setSession(session);
		this.tableName = tableName;
		this.onlyParamKeyBinding = onlyParamKeyBinding;
		this.parameters = parameters;
		this.primaryKeys = primaryKeys;
	}

	/**
	 * Query 생성자 (TABLE)
	 * 
	 * @param tableName
	 *            - 테이블명
	 * @param mode
	 *            - 쿼리 모드
	 * @param primaryKeys
	 *            - 프라이머리 키 배열
	 */
	public TableQueryImpl(Session session, String tableName, String[] primaryKeys, String[] processTargetColumns, Map parameters) {
		super.setSession(session);
		this.tableName = tableName;
		this.primaryKeys = primaryKeys;
		this.processTargetColumns = processTargetColumns;
		this.parameters = parameters;
	}

	/**
	 * Query 생성자 (TABLE)
	 * 
	 * @param tableName
	 *            - 테이블명
	 * @param primaryKeys
	 *            - 프라이머리 키 배열
	 * @param mode
	 *            - 쿼리 모드
	 * @param onlyParamKeyBinding
	 *            - 파라메터 맵속의 Key만 INSERT/UPDATE 컬럼으로 바인딩
	 */
	public TableQueryImpl(Session session, String tableName, String[] primaryKeys, boolean onlyParamKeyBinding, Map parameters) {
		super.setSession(session);
		this.tableName = tableName;
		this.primaryKeys = primaryKeys;
		this.onlyParamKeyBinding = onlyParamKeyBinding;
		this.parameters = parameters;
	}

	/**
	 * Query 실행을 위한 프로세서를 반환한다.
	 * 
	 * @return
	 */
	public TableQueryProcessor execute() {
		return new TableQueryProcessorImpl(this);
	}

	public boolean isOnlyParamKeyBinding() {
		return onlyParamKeyBinding;
	}

	public void setOnlyParamKeyBinding(boolean onlyParamKeyBinding) {
		this.onlyParamKeyBinding = onlyParamKeyBinding;
	}

	public String[] getPrimaryKeys() {
		return primaryKeys;
	}

	public void setPrimaryKeys(String[] primaryKeys) {
		this.primaryKeys = primaryKeys;
	}

	public String[] getProcessTargetColumns() {
		return processTargetColumns;
	}

	public void setProcessTargetColumns(String[] processTargetColumns) {
		this.processTargetColumns = processTargetColumns;
	}

	public String getTableName() {
		return tableName;
	}

	public void setTableName(String tableName) {
		this.tableName = tableName;
	}

	public Map getParameters() {
		return parameters;
	}

	public void setParameters(Map parameters) {
		this.parameters = parameters;
	}

	public void addParameter(String key, Object value) {
		if (parameters == null) {
			parameters = new HashMap();
		}
		parameters.put(key, value);
	}

	public void removeParameter(String key) {
		if (parameters != null) {
			parameters.remove(key);
		}
	}

}
