package org.jdesigner.platform.util;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.dom4j.Attribute;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.io.HTMLWriter;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;
import org.xml.sax.InputSource;

/**
 * Utility methods for working with dom4j Documents.
 * 
 * @author leesangboo
 */
public abstract class Dom4jUtils {

	private static boolean debug = false;

	/**
	 * Load an XML file into a dom4j Document. If error, return null. No validation is performed.
	 * 
	 * @param file
	 *            An XML file
	 * @return An XML Document containing the dom4j DOM, or null if unable to process the file.
	 * @exception DocumentException
	 *                If dom4j error
	 * @exception MalformedURLException
	 *                If error in file name
	 */
	public static Document getXmlDocument(File file) throws DocumentException, MalformedURLException {
		// No validation...
		SAXReader reader = new SAXReader(false);
		Document document = reader.read(file);
		return document;
	}

	/**
	 * Load XML into a dom4j Document. If error, return null. No validation is performed.
	 * 
	 * @param url
	 *            A URL to an XML document
	 * @return An XML Document containing the dom4j DOM, or null if unable to process the file.
	 * @exception DocumentException
	 *                If dom4j error
	 * @exception MalformedURLException
	 *                If error in the URL
	 */
	public static Document getXmlDocument(URL url) throws DocumentException, MalformedURLException {
		// No validation...
		SAXReader reader = new SAXReader(false);
		Document document = reader.read(url);
		return document;
	}

	public static Document getXmlDocument(InputStream is) throws DocumentException {
		// No validation...
		SAXReader reader = new SAXReader(false);
		Document document = reader.read(is);
		return document;
	}

	/**
	 * Load XML into a dom4j Document. If error, return null. No validation is performed.
	 * 
	 * @param s
	 *            A string representation of an XML document
	 * @return An XML Document containing the dom4j DOM, or null if unable to process the string.
	 * @exception DocumentException
	 *                If dom4j error
	 */
	public static Document getXmlDocument(String s) throws DocumentException {
		// No validation...
		SAXReader reader = new SAXReader(false);
		InputSource input = new InputSource(new StringReader(s));
		Document document = reader.read(input);
		return document;
	}

	/**
	 * return node text
	 * 
	 * @param doc
	 * @param path
	 * @return
	 */
	public static String getNodeText(Document doc, String path) {
		Node node = doc.selectSingleNode(path);
		if (node != null) {
			return node.getText();
		} else {
			return null;
		}
	}

	/**
	 * isEmpty returns true if neither the element, or any children recursively, have textual content or have a non-empty attribute
	 * 
	 * @param e
	 *            an Element to test
	 * @return true if Element and all children recursively are empty
	 */
	public static boolean isEmpty(Element e) {
		String t = e.getText();
		if ((t == null) || (t.trim().length() > 0)) {
			return false;
		}

		// is there a non-empty attribute?
		for (Iterator i = e.attributeIterator(); i.hasNext();) {
			Attribute a = (Attribute) i.next();
			String value = a.getValue();
			if ((value == null) || (value.trim().length() > 0)) {
				return false;
			}
		}

		// is there a non-empty child element?
		for (Iterator i = e.elementIterator(); i.hasNext();) {
			Element child = (Element) i.next();
			boolean isChildEmpty = isEmpty(child);
			if (!isChildEmpty) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Strip namespace information from a particular element within a string representation of an XML element (having arbitrary content and
	 * subelements). Textually remove all namespace (and all other attributes) from the first occurance of an element of <b> elementName</b> in the
	 * input string. This is done using a regex pattern replace
	 * 
	 * @param xml
	 *            Description of the Parameter
	 * @param elementName
	 *            Description of the Parameter
	 * @return Description of the Return Value
	 */
	public static String localizeXml(String xml, String elementName) {
		// Create a pattern to match the element start tag
		Pattern p = Pattern.compile("<" + elementName + ".+?>", Pattern.MULTILINE);

		// Create a matcher with an input string
		Matcher m = p.matcher(xml);
		return m.replaceFirst("<" + elementName + ">");
	}

	/**
	 * Remove nameSpaceInformation from the root element to facilitate xpath access
	 * 
	 * @param doc
	 *            Description of the Parameter
	 * @param rootElementName
	 *            Description of the Parameter
	 * @return Description of the Return Value
	 */
	public static Document localizeXml(Document doc, String rootElementName) {
		String localizedXml = localizeXml(doc.asXML(), rootElementName);
		try {
			return getXmlDocument(localizedXml);
		} catch (Exception e) {
			prtln("unable to localizeItemRecord(): " + e);
		}
		return null;
	}

	/**
	 * Insert nameSpaceInformation into the root element to make validation possible
	 * 
	 * @param s
	 *            Description of the Parameter
	 * @param rootElementName
	 *            Description of the Parameter
	 * @param nameSpaceInfo
	 *            Description of the Parameter
	 * @return Description of the Return Value
	 */
	public static String delocalizeDocStr(String s, String rootElementName, String nameSpaceInfo) {
		// Create a pattern to match the itemRecord element
		Pattern p = Pattern.compile("<" + rootElementName + "\\s*>", Pattern.MULTILINE);
		// Create a matcher with an input string
		Matcher m = p.matcher(s);
		return m.replaceFirst("<" + rootElementName + " " + nameSpaceInfo + ">");
	}

	/**
	 * Insert nameSpaceInformation into the root element
	 * 
	 * @param doc
	 *            Description of the Parameter
	 * @param rootElementName
	 *            Description of the Parameter
	 * @param nameSpaceInfo
	 *            Description of the Parameter
	 * @return Description of the Return Value
	 */
	public static Document delocalizeXml(Document doc, String rootElementName, String nameSpaceInfo) {
		// Element root = doc.getRootElement();
		// String rootName = root.getName();
		String namespaceXml = delocalizeDocStr(doc.asXML(), rootElementName, nameSpaceInfo);
		try {
			return getXmlDocument(namespaceXml);
		} catch (Exception e) {
			prtln("unable to delocalizeItemRecordDoc(): " + e);
		}
		return null;
	}

	/**
	 * Gets the nameSpaceInfo associated with the root element of a Document. Returns all contents of the root Element except the element name.
	 * 
	 * @param doc
	 *            Description of the Parameter
	 * @param rootElementName
	 *            Description of the Parameter
	 * @return The nameSpaceInfo value
	 */
	public static String getNameSpaceInfo(Document doc, String rootElementName) {
		String xml = doc.asXML();
		Pattern p = Pattern.compile("<" + rootElementName + "(.+?)>", Pattern.MULTILINE);
		Matcher m = p.matcher(xml);

		if (!m.find()) {
			prtln("root Element not found!?: " + p.toString());
			return "";
		} else {
			return m.group(1);
		}
	}

	/**
	 * Writes {@link org.dom4j.Document} to File
	 * 
	 * @param doc
	 *            Description of the Parameter
	 * @param out
	 *            Description of the Parameter
	 * @exception Exception
	 *                Description of the Exception
	 */
	public static void writeDocToFile(Document doc, File out) throws Exception {
		writeDocToFile(doc, out, null);
	}

	/**
	 * write formated xml to file to faciliate human-readibility
	 * 
	 * @param doc
	 *            Description of the Parameter
	 * @param out
	 *            Description of the Parameter
	 * @exception Exception
	 *                Description of the Exception
	 */
	public static void writePrettyDocToFile(Document doc, File out) throws Exception {
		OutputFormat format = OutputFormat.createPrettyPrint();
		writeDocToFile(doc, out, format);
	}

	/**
	 * Write XML Document to disk using specified OutputFormat
	 * 
	 * @param doc
	 *            Description of the Parameter
	 * @param out
	 *            Description of the Parameter
	 * @param format
	 *            Description of the Parameter
	 * @exception Exception
	 *                Description of the Exception
	 */
	public static void writeDocToFile(Document doc, File out, OutputFormat format) throws Exception {
		String encoding = "UTF-8";
		if (format == null) {
			format = new OutputFormat();
		}
		OutputStreamWriter fileWriter = new OutputStreamWriter(new FileOutputStream(out), encoding);
		XMLWriter writer = new XMLWriter(fileWriter, format);
		writer.write(doc);
		fileWriter.close();
	}

	/**
	 * Gets an XMLWriter object
	 * 
	 * @return The xMLWriter value
	 */
	public static XMLWriter getXMLWriter() {
		OutputFormat format = new OutputFormat("  ", false);
		XMLWriter writer = null;
		try {
			writer = new XMLWriter(System.out, format);
		} catch (Exception e) {
			prtln("getXMLWriter(): " + e);
		}
		return writer;
	}

	/**
	 * Formats an {@link org.dom4j.Node} as a printable string
	 * 
	 * @param node
	 *            the Node to display
	 * @return a nicley-formatted String representation of the Node.
	 */
	public static String prettyPrint(Node node) {
		StringWriter sw = new StringWriter();
		OutputFormat format = OutputFormat.createPrettyPrint();

		format.setXHTML(true);
		// Default is false, this produces XHTML
		HTMLWriter ppWriter = new HTMLWriter(sw, format);
		try {
			ppWriter.write(node);
			ppWriter.flush();
		} catch (Exception e) {
			return ("Pretty Print Failed");
		}
		return sw.toString();
	}

	// ================================================================

	/**
	 * Logger hook
	 * 
	 * @param msg
	 *            Message to be logged.
	 */
	public void log(String msg) {
		prtln(msg);
	}

	/**
	 * Return a string for the current time and date, sutiable for display in log files and output to standout:
	 * 
	 * @return The dateStamp value
	 */
	protected final static String getDateStamp() {
		return new SimpleDateFormat("MMM d, yyyy h:mm:ss a zzz").format(new Date());
	}

	/**
	 * Output a line of text to error out, with datestamp.
	 * 
	 * @param s
	 *            The text that will be output to error out.
	 */
	protected final static void prtlnErr(String s) {
		System.err.println(getDateStamp() + " " + s);
	}

	/**
	 * Output a line of text to standard out, with datestamp, if debug is set to true.
	 * 
	 * @param s
	 *            The String that will be output.
	 */
	protected final static void prtln(String s) {
		if (debug) {
			System.out.println(getDateStamp() + " " + s);
		}
	}

	/**
	 * Sets the debug attribute of the object
	 * 
	 * @param db
	 *            The new debug value
	 */
	public static void setDebug(boolean db) {
		debug = db;
	}

}
