/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jdesigner.platform.web.filter;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.mozilla.javascript.ErrorReporter;
import org.mozilla.javascript.EvaluatorException;

import com.yahoo.platform.yui.compressor.CssCompressor;
import com.yahoo.platform.yui.compressor.JavaScriptCompressor;

/**
 * JsCssMinifyFilter
 * 
 */
public class JsCssMinifyFilter implements Filter {

	private static Log log = LogFactory.getLog(JsCssMinifyFilter.class);

	// private FilterConfig config;

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	public void init(FilterConfig filterConfig) throws ServletException {
		// this.config = filterConfig;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
	 */
	public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {

		HttpServletRequest originalRequest = (HttpServletRequest) request;

		HttpServletResponse originalResponse = (HttpServletResponse) response;

		// Create the response wrapper
		MinifiedResponse minifiedResponse = new MinifiedResponse(originalResponse);

		// Continue the chain - get the contents
		chain.doFilter(originalRequest, minifiedResponse);

		// The data
		String data = new String(minifiedResponse.getData());

		// Create the reader
		StringReader reader = new StringReader(data);
		// Create the writer
		StringWriter writer = new StringWriter();

		String requestURI = originalRequest.getRequestURI();

		try {
			// Do JS minification
			if (requestURI.endsWith(".js")) {
				log.debug(requestURI + " is minify javascript filter.");
				minifyJS(reader, writer);
			}
			// Do Css minification
			else if (requestURI.endsWith(".css")) {
				log.debug(requestURI + " is minify style class filter.");
				minifyCSS(reader, writer);
			}
			// Else - we can't do any other minification
			else {
				writer.write(data);
			}

		} catch (Exception ex) {
			// We do a generic exception handling
			// Log error
			log.error("Error in minifying : " + requestURI, ex);
			// Write the data as it is
			writer.write(data);
		}

		// Write the data to the original response
		// Get the data from the writer (Compressed data)
		String compressed = writer.toString();

		// Set the content length correctly
		originalResponse.setContentLength(compressed.length());

		// Write the data
		originalResponse.getOutputStream().write(compressed.getBytes());

	}

	/**
	 * @param reader
	 * @param writer
	 * @throws IOException
	 */
	private void minifyCSS(StringReader reader, StringWriter writer) throws IOException {
		CssCompressor cssCompressor = new CssCompressor(reader);
		cssCompressor.compress(writer, -1);
	}

	/**
	 * @param reader
	 * @param writer
	 */
	private void minifyJS(StringReader reader, StringWriter writer) throws IOException {
		JavaScriptCompressor compressor = new JavaScriptCompressor(reader, new ErrorReporter() {

			public void warning(String message, String sourceName, int line, String lineSource, int lineOffset) {
				if (line < 0) {
					log.warn("[WARNING] " + message);
				} else {
					log.warn("[WARNING] " + line + ':' + lineOffset + ':' + message);
				}
			}

			public void error(String message, String sourceName, int line, String lineSource, int lineOffset) {
				if (line < 0) {
					log.error("[ERROR] " + message);
				} else {
					log.error("[ERROR] " + line + ':' + lineOffset + ':' + message);
				}
			}

			public EvaluatorException runtimeError(String message, String sourceName, int line, String lineSource, int lineOffset) {
				error(message, sourceName, line, lineSource, lineOffset);
				return new EvaluatorException(message);
			}
		});

		// Does the compression
		compressor.compress(writer, -1, true, false, true, false);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.Filter#destroy()
	 */
	public void destroy() {

	}

	/**
	 * A response wrapper, the content is saved to the output buffer, the stream returned is ServletOutputStreamWrapper
	 * 
	 * @author Vinu Varghese (Email: vinu@x-minds.org)
	 * 
	 */
	class MinifiedResponse extends HttpServletResponseWrapper {

		private ByteArrayOutputStream output = new ByteArrayOutputStream();

		/**
		 * @param response
		 */
		public MinifiedResponse(HttpServletResponse response) {
			super(response);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.servlet.ServletResponseWrapper#getOutputStream()
		 */

		public ServletOutputStream getOutputStream() throws IOException {
			return new ServletOutputStreamWrapper(this.output);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.servlet.ServletResponseWrapper#getWriter()
		 */

		public PrintWriter getWriter() throws IOException {
			PrintWriter localPrintWriter;
			try {
				localPrintWriter = new PrintWriter(new OutputStreamWriter(getOutputStream(), "UTF-8"), true);
			} catch (Exception localException) {
				localPrintWriter = new PrintWriter(getOutputStream(), true);
			}

			return localPrintWriter;
		}

		/**
		 * Returns the data
		 * 
		 * @return
		 */
		public byte[] getData() {
			try {
				output.flush();
			} catch (IOException e) {
				// ignore
			}
			return output.toByteArray();
		}

	}

	/**
	 * Output stream wrapper I have overriden the write method, is that enough?
	 * 
	 * @author Vinu Varghese (Email: vinu@x-minds.org)
	 * 
	 */
	class ServletOutputStreamWrapper extends ServletOutputStream {

		private ByteArrayOutputStream output;

		/**
		 * 
		 */
		public ServletOutputStreamWrapper(ByteArrayOutputStream output) {
			this.output = output;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.io.OutputStream#write(int)
		 */
		public void write(int b) throws IOException {
			output.write(b);
		}

	}

}
